@@ -227,7 +227,7 @@ class ViewContainer(Gtk.Stack):
         if not vsbl_range or self.old_vsbl_range == vsbl_range:
             return
         self.old_vsbl_range = vsbl_range
-        self.on_scroll_event(self.view)
+        GLib.idle_add(self.on_scroll_event, self.view)
 
     @log
     def on_scroll_event(self, widget, event=None):
@@ -322,7 +322,7 @@ class ViewContainer(Gtk.Stack):
         self.header_bar._select_button.clicked()
 
     @log
-    def get_selected_track_uris(self, callback):
+    def get_selected_tracks(self, callback):
         callback([])
 
 
@@ -390,13 +390,13 @@ class Albums(ViewContainer):
             GLib.idle_add(grilo.populate_albums, self._offset, self._add_item)
 
     @log
-    def get_selected_track_uris(self, callback):
+    def get_selected_tracks(self, callback):
         if self.header_bar._state == ToolbarState.CHILD_VIEW:
-            uris = []
+            items = []
             for path in self._albumWidget.view.get_selection():
                 _iter = self._albumWidget.model.get_iter(path)
-                uris.append(self._albumWidget.model.get_value(_iter, 5).get_url())
-            callback(uris)
+                items.append(self._albumWidget.model.get_value(_iter, 5))
+            callback(items)
         else:
             self.items_selected = []
             self.items_selected_callback = callback
@@ -416,7 +416,7 @@ class Albums(ViewContainer):
     @log
     def _add_selected_item(self, source, param, item, remaining=0, data=None):
         if item:
-            self.items_selected.append(item.get_url())
+            self.items_selected.append(item)
         if remaining == 0:
             if self.albums_index < len(self.albums_selected):
                 self._get_selected_album_songs()
@@ -578,12 +578,13 @@ class Songs(ViewContainer):
 
     @log
     def populate(self):
+        self._init = True
         if grilo.tracker:
             GLib.idle_add(grilo.populate_songs, self._offset, self._add_item)
 
     @log
-    def get_selected_track_uris(self, callback):
-        callback([self._model.get_value(self._model.get_iter(path), 5).get_url()
+    def get_selected_tracks(self, callback):
+        callback([self._model.get_value(self._model.get_iter(path), 5)
                   for path in self.view.get_selection()])
 
 
@@ -640,10 +641,10 @@ class Artists (ViewContainer):
             self._artists[_("All Artists").casefold()] =\
                 {'iter': self._allIter, 'albums': [], 'widget': None}
             selection.select_path(self._model.get_path(self._allIter))
-            self.view.emit('item-activated', '0',
-                           self._model.get_path(self._allIter))
         self._init = True
         self.populate()
+        self.view.emit('item-activated', '0',
+                       self._model.get_path(self._allIter))
 
     @log
     def _add_list_renderers(self):
@@ -753,7 +754,7 @@ class Artists (ViewContainer):
             self._on_changes_pending()
 
     @log
-    def get_selected_track_uris(self, callback):
+    def get_selected_tracks(self, callback):
         self.items_selected = []
         self.items_selected_callback = callback
         self.albums_index = 0
@@ -780,7 +781,7 @@ class Artists (ViewContainer):
     @log
     def _add_selected_item(self, source, param, item, remaining=0, data=None):
         if item:
-            self.items_selected.append(item.get_url())
+            self.items_selected.append(item)
         if remaining == 0:
             if self.albums_index < len(self.albums_selected):
                 self._get_selected_album_songs()
@@ -789,7 +790,6 @@ class Artists (ViewContainer):
 
 
 class Playlist(ViewContainer):
-    playlists_list = playlists.get_playlists()
 
     @log
     def __init__(self, window, player):
@@ -982,31 +982,49 @@ class Playlist(ViewContainer):
         return False
 
     @log
-    def _add_playlist_item(self, item):
-        _iter = self.playlists_model.append()
-        self.playlists_model.set(_iter, [2], [item])
+    def _add_playlist_item(self, source, param, item, remaining=0, data=None):
+        self._add_playlist_item_to_model(item)
+
+    @log
+    def _add_playlist_item_to_model(self, item):
+        if not item:
+            return
+        _iter = self.playlists_model.insert_with_valuesv(
+            -1,
+            [2, 5],
+            [albumArtCache.get_media_title(item), item])
+        if self.playlists_model.iter_n_children(None) == 1:
+            _iter = self.playlists_model.get_iter_first()
+            selection = self.playlists_sidebar.get_generic_view().get_selection()
+            selection.select_iter(_iter)
+            self.playlists_sidebar.emit('item-activated', '0',
+                                        self.playlists_model.get_path(_iter))
 
     @log
     def _on_item_activated(self, widget, id, path):
         _iter = self._model.get_iter(path)
         if self._model.get_value(_iter, 8) != self.errorIconName:
-            self.player.set_playlist('Playlist', self.current_playlist, self._model, _iter, 5)
+            self.player.set_playlist(
+                'Playlist', self.current_playlist.get_id(),
+                self._model, _iter, 5
+            )
             self.player.set_playing(True)
 
     @log
     def _on_playlist_activated(self, widget, item_id, path):
         _iter = self.playlists_model.get_iter(path)
-        playlist = self.playlists_model.get_value(_iter, 2)
+        playlist_name = self.playlists_model.get_value(_iter, 2)
+        playlist = self.playlists_model.get_value(_iter, 5)
 
         if self.current_playlist == playlist:
             return
 
         self.current_playlist = playlist
-        self.name_label.set_text(playlist)
+        self.name_label.set_text(playlist_name)
 
         # if the active queue has been set by this playlist,
         # use it as model, otherwise build the liststore
-        cached_playlist = self.player.running_playlist('Playlist', playlist)
+        cached_playlist = self.player.running_playlist('Playlist', playlist_name)
         if cached_playlist:
             self._model = cached_playlist
             currentTrack = self.player.playlist.get_iter(self.player.currentTrack.get_path())
@@ -1030,7 +1048,7 @@ class Playlist(ViewContainer):
                 GObject.TYPE_BOOLEAN
             )
             self.view.set_model(self._model)
-            playlists.parse_playlist(playlist, self._add_item)
+            GLib.idle_add(grilo.populate_playlist_songs, playlist, self._add_item)
             self.songs_count = 0
             self._update_songs_count()
 
@@ -1043,15 +1061,15 @@ class Playlist(ViewContainer):
         if not item:
             return
         self._offset += 1
-        item.set_title(albumArtCache.get_media_title(item))
+        title = albumArtCache.get_media_title(item)
+        item.set_title(title)
         artist = item.get_string(Grl.METADATA_KEY_ARTIST)\
             or item.get_author()\
             or _("Unknown Artist")
         _iter = model.insert_with_valuesv(
             -1,
             [2, 3, 5, 8, 9, 10],
-            [albumArtCache.get_media_title(item),
-             artist, item, self.nowPlayingIconName, False, False])
+            [title, artist, item, self.nowPlayingIconName, False, False])
         self.player.discover_item(item, self._on_discovered, _iter)
         self.songs_count += 1
         self._update_songs_count()
@@ -1093,13 +1111,13 @@ class Playlist(ViewContainer):
             self.playlists_sidebar.emit('item-activated', '0',
                                         self.playlists_model.get_path(iter_next))
 
-        playlist = self.playlists_model.get_value(_iter, 2)
+        playlist = self.playlists_model.get_value(_iter, 5)
         playlists.delete_playlist(playlist)
         self.playlists_model.remove(_iter)
 
     @log
-    def _on_playlist_created(self, playlists, name):
-        self._add_playlist_item(name)
+    def _on_playlist_created(self, playlists, item):
+        self._add_playlist_item_to_model(item)
         if self.playlists_model.iter_n_children(None) == 1:
             _iter = self.playlists_model.get_iter_first()
             selection = self.playlists_sidebar.get_generic_view().get_selection()
@@ -1108,30 +1126,37 @@ class Playlist(ViewContainer):
                                         self.playlists_model.get_path(_iter))
 
     @log
-    def _on_song_added_to_playlist(self, playlists, name, item):
-        if name == self.current_playlist:
+    def _on_song_added_to_playlist(self, playlists, playlist, item):
+        if self.current_playlist and \
+           playlist.get_id() == self.current_playlist.get_id():
             self._add_item_to_model(item, self._model)
         else:
-            cached_playlist = self.player.running_playlist('Playlist', name)
+            cached_playlist = self.player.running_playlist(
+                'Playlist', playlist.get_id()
+            )
             if cached_playlist and cached_playlist != self._model:
                 self._add_item_to_model(item, cached_playlist)
 
     @log
-    def _on_song_removed_from_playlist(self, playlists, name, uri):
-        cached_playlist = self.player.running_playlist('Playlist', name)
-        if name == self.current_playlist:
+    def _on_song_removed_from_playlist(self, playlists, playlist, item):
+        cached_playlist = self.player.running_playlist(
+            'Playlist', playlist.get_id()
+        )
+        if self.current_playlist and \
+           playlist.get_id() == self.current_playlist.get_id():
             model = self._model
+        elif cached_playlist and cached_playlist != self._model:
+            model = cached_playlist
         else:
-            if cached_playlist and cached_playlist != self._model:
-                model = cached_playlist
-            else:
-                return
+            return
 
         update_playing_track = False
         for row in model:
-            if row[5].get_url() == uri:
+            if row[5].get_id() == item.get_id():
                 # Is the removed track now being played?
-                if name == self.current_playlist and cached_playlist == model:
+                if self.current_playlist and \
+                   playlist.get_id() == self.current_playlist.get_id() and \
+                   cached_playlist == model:
                     if self.player.currentTrack is not None:
                         currentTrackpath = self.player.currentTrack.get_path().to_string()
                         if row.path is not None and row.path.to_string() == currentTrackpath:
@@ -1151,7 +1176,7 @@ class Playlist(ViewContainer):
 
                     self.iter_to_clean = None
                     self.update_model(self.player, model, nextIter)
-                    self.player.set_playlist('Playlist', name, model, nextIter, 5)
+                    self.player.set_playlist('Playlist', playlist.get_id(), model, nextIter, 5)
                     self.player.set_playing(True)
 
                 # Update songs count
@@ -1161,18 +1186,13 @@ class Playlist(ViewContainer):
 
     @log
     def populate(self):
-        for item in sorted(self.playlists_list):
-            self._add_playlist_item(item)
-        if len(self.playlists_list):
-            _iter = self.playlists_model.get_iter_first()
-            selection = self.playlists_sidebar.get_generic_view().get_selection()
-            selection.select_iter(_iter)
-            self.playlists_sidebar.emit('item-activated', '0',
-                                        self.playlists_model.get_path(_iter))
+        if grilo.tracker:
+            GLib.idle_add(grilo.populate_playlists, self._offset,
+                          self._add_playlist_item)
 
     @log
-    def get_selected_track_uris(self, callback):
-        callback([self._model.get_value(self._model.get_iter(path), 5).get_url()
+    def get_selected_tracks(self, callback):
+        callback([self._model.get_value(self._model.get_iter(path), 5)
                   for path in self.view.get_selection()])
 
 
@@ -1211,7 +1231,6 @@ class Search(ViewContainer):
         if self.get_visible_child() == self._artistAlbumsWidget:
             self._artistAlbumsWidget.destroy()
             self._artistAlbumsWidget = None
-        self.window._stack.set_visible_child(self.window.prev_view)
         self.set_visible_child(self._grid)
 
     @log
@@ -1373,19 +1392,19 @@ class Search(ViewContainer):
         pass
 
     @log
-    def get_selected_track_uris(self, callback):
+    def get_selected_tracks(self, callback):
         if self.get_visible_child() == self._albumWidget:
-            uris = []
+            items = []
             for path in self._albumWidget.view.get_selection():
                 _iter = self._albumWidget.model.get_iter(path)
-                uris.append(self._albumWidget.model.get_value(_iter, 5).get_url())
-            callback(uris)
+                items.append(self._albumWidget.model.get_value(_iter, 5))
+            callback(items)
         elif self.get_visible_child() == self._artistAlbumsWidget:
-            uris = []
+            items = []
             for row in self._artistAlbumsWidget.model:
                 if row[6]:
-                    uris.append(row[5].get_url())
-            callback(uris)
+                    items.append(row[5])
+            callback(items)
         else:
             self.items_selected = []
             self.items_selected_callback = callback
@@ -1413,7 +1432,7 @@ class Search(ViewContainer):
     @log
     def _add_selected_albums_songs(self, source, param, item, remaining=0, data=None):
         if item:
-            self.items_selected.append(item.get_url())
+            self.items_selected.append(item)
         if remaining == 0:
             if self.albums_index < len(self.albums_selected):
                 self._get_selected_albums_songs()
@@ -1447,7 +1466,7 @@ class Search(ViewContainer):
     @log
     def _add_selected_artists_albums_songs(self, source, param, item, remaining=0, data=None):
         if item:
-            self.items_selected.append(item.get_url())
+            self.items_selected.append(item)
         if remaining == 0:
             if self.artists_albums_index < len(self.artists_albums_selected):
                 self._get_selected_artists_albums_songs()
@@ -1456,7 +1475,7 @@ class Search(ViewContainer):
 
     @log
     def _get_selected_songs(self):
-        self.items_selected.extend([self._model[child_path][5].get_url()
+        self.items_selected.extend([self._model[child_path][5]
                                     for child_path in [self.filter_model.convert_path_to_child_path(path)
                                                        for path in self.view.get_selection()]
                                     if self._model[child_path][11] == 'song'])
